%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This code allow to run the inference algorithms (either the prosposed
%   SMC sampler or the Importance sampling algorithm) after the data
%   has been generated using the GenerationData.m matlab file
%
%   This code will generate a .mat file (in the SavedData directory) 
%   that contains most of the important ouput of the Monte Carlo 
%   algorithms in order to analyze its accuracy.

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Matlab Codes associated to the paper
%       ``A Bayesian Perspective on Multiple Source
%        Localization in Wireless Sensor Network'' 
% by T.L.T Nguyen, F. Septier, H. Rajaona, G.W. Peters, I. Nevat,
%       Y. Delignon
%  
%       Date: Sep. 2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
close all

addpath('Distributions/')
addpath('SMCSampler/')
addpath('ISSampler/')

NbMCruns=10; % Number of scenario to be analyzed
Kmax=6; % Maximum number of sources to be tested 
AllModelToTry=1:Kmax; % This is the model (number of targets) to be tested

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters of the inference Algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ImportanceSampling=0; % Choice of the sampler: 0 - SMC Sampler | 1 - Importance Sampler
NbIterationAveragedforIS=24; % Average of the SMC Sampler nb iterations in order to be fair in the comparison of the IS NbpartIS=NbPartSMC x NbIteration
PercentageCESS=0.9; % Threshold of the CESS for the adaptive cooling schedule of the SMC sampler
CurrentNbParticles=100; % Number of Particles for each iterations of the SMC samplers
CurrentNbMCMCMoves=5; % Number of MCMC move in the SMC sampler algorithm



NbTime=700;% Set Maximum number of iterations of the SMC sampler - Used mainly for initializing the variables required in the algorithm




for mAllModelToTry=AllModelToTry % Run a Monte-Carlo algorithm over each model 
    
    % Variable that will adjust the number of particles in the IS in order
    % to have the computational cost than the proposed SMC sampler
    AverageSMCNbIterations=18; % This variable is to adjust according to the number of SMC sampler algorithm for this scenario
    NbIterationAveragedforIS=mAllModelToTry*CurrentNbMCMCMoves*AverageSMCNbIterations;
    
    % Scenario (generated using GenerationData.m to be Loaded for which the inference will be performed
    CommonStringDataToLoad=['TargetScenario_NbSensors=100_QuantNbLevel=8_NoiseVar=1_NumMcRun=']; 
    
    % Matlab code that launch the inference algorithm on the NbMCruns
    % different scenarios
    TargetLocalization
    
    % Save the results
    if ImportanceSampling==0
        FileNameToSave=['Results_SMC_',CommonStringDataToLoad,'_Model',num2str(mAllModelToTry),'_PercentageCESS=',num2str(PercentageCESS),'_NbPart=',num2str(CurrentNbParticles),'_NbMCMCMove=',num2str(CurrentNbMCMCMoves),'_NbBlock=',num2str(CurrentNbBlockMWG),'_AdaptiveTemperature','.mat'];
    else
        FileNameToSave=['Results_IS_',CommonStringDataToLoad,'_Model',num2str(mAllModelToTry),'_NbPartTotal=',num2str(NbIterationAveragedforIS*CurrentNbParticles),'.mat'];
    end    
    
    eval(['save SavedData/',FileNameToSave])
end

 