%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This code is called by SimulationLauncher.m and run the algorithm 
% on the different realization of the scenarios that are under study
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Matlab Codes associated to the paper
%       ``A Bayesian Perspective on Multiple Source
%        Localization in Wireless Sensor Network'' 
% by T.L.T Nguyen, F. Septier, H. Rajaona, G.W. Peters, I. Nevat,
%       Y. Delignon
%  
%       Date: Sep. 2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



clear ResultsSMC;
TimeTemp=[0:NbTime]/NbTime;
Temperature=zeros(1,NbTime+1);


Target.Nb=mAllModelToTry; % Number of target is given by the model order
Target.Dim=2*Target.Nb;

ResultsSMC.SavedLogBayesianEvidence=zeros(1,NbMCruns);
ResultsSMC.SavedEstimateBeta.Mean=zeros(Target.Dim,NbMCruns);
ResultsSMC.SavedEstimateBeta.Cov=zeros(Target.Dim,Target.Dim,NbMCruns);

ResultsSMC.MCruns=[];


for nMCruns=1:NbMCruns % Loop over all the MC realization under study
    
       
        disp(['Scenario ',num2str(nMCruns),' of ',num2str(NbMCruns)])

        % Load the generated data - Observations
        eval(['load SavedData/',CommonStringDataToLoad,num2str(nMCruns),'.mat']) % Load Corresponding New Observations Set

        if nMCruns==1
            TrueTargetPosition=zeros(size(Target.Position,1),NbMCruns);
        end
        % Definition of the number of Target for the algorithm that depends on
        % the model order to test
        Target.Nb=mAllModelToTry;
        Target.Dim=2*Target.Nb;
        Target.PriorCov=Target.PriorCov(1:1,1:1)*eye(Target.Dim);
        Target.PriorMean=repmat(Target.PriorMean(1:2,1),Target.Nb,1);    

        
        CurrentNbBlockMWG=mAllModelToTry; % Number of Blocks equal to the number of sources
        
        if ImportanceSampling==0 % Run SMC sampler
            clear SMCSampler;
            SMCSampler.Temperature=Temperature;% Initialization
            SMCSampler.NbPart=CurrentNbParticles; % Number of Particle
            SMCSampler.ResamplingEachIteration=0;  % 1 - To force resampling every iteration !!       
            [SMCSampler]=SMCSamplerAlgo(SMCSampler,Target,CurrentNbMCMCMoves,CurrentNbBlockMWG,Sensors,PercentageCESS);%,AdaptiveCoolingStrategy,PosteriorGrid,Xgrid,Ygrid);
        else % Run IS sampler
            SMCSampler.NbPart=CurrentNbParticles*NbIterationAveragedforIS; % Adjust the number of particles to have the cost as the SMC sampler
            [SMCSampler]=ImportanceSampler(SMCSampler,Target,CurrentNbMCMCMoves,CurrentNbBlockMWG,Sensors,PercentageCESS);%,AdaptiveCoolingStrategy,PosteriorGrid,Xgrid,Ygrid);
        end     
       
        

        % Saved Different Output of the algorithm for future analysis !

        ResultsSMC.SavedLogBayesianEvidence(1,nMCruns)=SMCSampler.LogBayesEvidence;% Log  Evidence;
        ResultsSMC.SavedEstimateBeta.Mean(:,nMCruns)=SMCSampler.Estimate.Mean; % MMSE Estimate (no relabeling)
        
        ResultsSMC.SavedEstimateBeta.Cov(:,:,nMCruns)=SMCSampler.Estimate.Cov; % Posterior Cov Estimate
        
        
        % Save all particles and weights of the last iteration
        
        ResultsSMC.MCruns(nMCruns).SavedNess=SMCSampler.EffectiveSampleSize; % ESS 
        ResultsSMC.SavedParticles.Beta(:,:,nMCruns)=[SMCSampler.Iter(end).Particles(:).Beta]; % Beta Correspond to the position
        ResultsSMC.SavedParticles.LogPower(:,:,nMCruns)=[SMCSampler.Iter(end).Particles(:).LogPower]; % LogPower corresponds to the log Transmitted power of the sources
        ResultsSMC.SavedParticles.NormalizedWeights(1,:,nMCruns)=SMCSampler.NormalizedWeights(:,:,SMCSampler.NumIter);
        
        
        ResultsSMC.MCruns(nMCruns).Temperature=SMCSampler.Temperature(1:SMCSampler.NumIter); % Temperature schedule that has been used (Adaptive CESS)
        
        ResultsSMC.MCruns(nMCruns).NbIter=SMCSampler.NumIter; % Number of Iteration of the algorithms - Used to adapt the number of IS particles

        TrueTargetPosition(:,nMCruns)=Target.Position; % Save the true position of the source
        TrueTargetPower(:,nMCruns)=Target.Power; % Save the true value of the transmitted power
    
end

% % % % % 
% % % % % 
% % % % % % 
% % % % % % if strcmp(AdaptiveCoolingStrategy,'ADAMCESS')
% % % % % %     Temperature=zeros(1,NbTime+1);%(exp(GammaTemp(IndiceRenyiOptimal-IndiceRenyiRespectToOptimal).*(TimeTemp))-1)./(exp(GammaTemp(IndiceRenyiOptimal-IndiceRenyiRespectToOptimal))-1);
% % % % % % else
% % % % % %     TimeTemp=[0:PercentageCESS]/PercentageCESS;
% % % % % %     NbGamma=100; % TO BE REMMOVED
% % % % % %     GammaTemp=linspace(1e-10,30,NbGamma); % TO BE REOMVED !!!
% % % % % %     Temperature=(exp(GammaTemp(IndiceRenyiOptimal-IndiceRenyiRespectToOptimal).*(TimeTemp))-1)./(exp(GammaTemp(IndiceRenyiOptimal-IndiceRenyiRespectToOptimal))-1);
% % % % % % end
% % % % % 
% % % % % 
% % % % % 
% % % % % %IndiceRenyiRespectToOptimal=0;%0;%-18;%0;%IRenyi-1;
% % % % % 
% % % % % 
% % % % % %Temperature=zeros(1,NbTime+1);%(exp(GammaTemp(IndiceRenyiOptimal-IndiceRenyiRespectToOptimal).*(TimeTemp))-1)./(exp(GammaTemp(IndiceRenyiOptimal-IndiceRenyiRespectToOptimal))-1);
% % % % % 
% % % % % 
% % % % % %----------------------------
% % % % % %Parameters of the SMC Sampler
% % % % % %----------------------------
% % % % % %CurrentNbParticles=250;
% % % % % 
% % % % % %Target.Dim=2;
% % % % % Target.Nb=ModelToTry;
% % % % % Target.Dim=2*Target.Nb;
% % % % % 
% % % % % ResultsSMC.SavedLogBayesianEvidence=zeros(1,NbMCruns);
% % % % % ResultsSMC.SavedEstimateBeta.Mean=zeros(Target.Dim,NbMCruns);
% % % % % ResultsSMC.SavedEstimateBeta.Cov=zeros(Target.Dim,Target.Dim,NbMCruns);
% % % % % 
% % % % % ResultsSMC.MCruns=[];
% % % % % 
% % % % % 
% % % % % PosteriorGrid=[];
% % % % % Xgrid=[];
% % % % % Ygrid=[];
% % % % % for nMCruns=1:NbMCruns
% % % % %        
% % % % %     nMCruns
% % % % %         
% % % % %        %eval(['load SavedData/',CommonStringDataToLoad,num2str(nMCruns),'.mat']) % Load Corresponding New Observations Set
% % % % %         eval(['load SavedData/',CommonStringDataToLoad,num2str(nMCruns),'.mat']) % Load Corresponding New Observations Set
% % % % %         % eval(['load SavedData/',CommonStringDataToLoad,num2str(19),'.mat']) % Load Corresponding New Observations Set
% % % % %         SameDataSet=1;%In order to do not do again Moment Matching by IS to save computation
% % % % %         
% % % % %         if nMCruns==1
% % % % %             TrueTargetPosition=zeros(size(Target.Position,1),NbMCruns);
% % % % %         end
% % % % %         Target.Nb=ModelToTry;
% % % % %         Target.Dim=2*Target.Nb;
% % % % %         Target.PriorCov=Target.PriorCov(1:1,1:1)*eye(Target.Dim);
% % % % %         Target.PriorMean=repmat(Target.PriorMean(1:2,1),Target.Nb,1);
% % % % %        
% % % % %         
% % % % %         if strcmp(AdaptiveCoolingStrategy,'PROPOSED') % In order to launch that
% % % % %             
% % % % %             % SKIP OPTIMAL COOLING SCHEDULE !!!
% % % % %              NbTimeSaved=NbTime;
% % % % %             NbTime=PercentageCESS;           
% % % % %             if SameDataSet==0
% % % % %                 VarianceSMConGammaGrid;
% % % % %                 [C,IndiceRenyiOptimal]=min(VarianceSMC.GaussianApprox); 
% % % % %             else
% % % % %                 if nMCruns==1
% % % % %                     VarianceSMConGammaGrid;
% % % % %                     [C,IndiceRenyiOptimal]=min(VarianceSMC.GaussianApprox); 
% % % % %                 end
% % % % %             end
% % % % %             NbTime=NbTimeSaved;
% % % % %             
% % % % % 
% % % % %             %IndiceRenyiOptimal=3;
% % % % %             IndiceRenyiOptimal
% % % % %             
% % % % %             % For ce to a specific value
% % % % %             %IndiceRenyiOptimal=10;
% % % % %             
% % % % %             
% % % % %             if (IndiceRenyiOptimal-IndiceRenyiRespectToOptimal)<=0
% % % % %                 IndiceRenyiRespectToOptimal=IndiceRenyiOptimal-1;
% % % % %             end
% % % % %                 
% % % % %             TimeTemp=[0:PercentageCESS]/PercentageCESS;
% % % % %             Temperature=(exp(GammaTemp(IndiceRenyiOptimal-IndiceRenyiRespectToOptimal).*(TimeTemp))-1)./(exp(GammaTemp(IndiceRenyiOptimal-IndiceRenyiRespectToOptimal))-1);            
% % % % %         end
% % % % % 
% % % % %         % -- SMC Sampler with Adaptive MH within Gibbs
% % % % %         %SMCSamplerv1
% % % % %         clear SMCSampler;
% % % % %         SMCSampler.Temperature=Temperature;% Best Ones Exponential
% % % % %         SMCSampler.NbPart=CurrentNbParticles;
% % % % %         SMCSampler.ResamplingEachIteration=0;    % To force resampling every iteration !!
% % % % % 
% % % % %         %SMCSamplerv1
% % % % % %        [SMCSampler]=SMCSamplerv1(SMCSampler,Target,CurrentNbMCMCMoves,CurrentNbBlockMWG,Sensors,PercentageCESS,AdaptiveCoolingStrategy,PosteriorGrid,Xgrid,Ygrid);
% % % % %         %[SMCSampler]=SMCSamplerPowerv1(SMCSampler,Target,CurrentNbMCMCMoves,CurrentNbBlockMWG,Sensors,PercentageCESS,AdaptiveCoolingStrategy,PosteriorGrid,Xgrid,Ygrid);
% % % % %         
% % % % %         if ImportanceSampling==0
% % % % %             [SMCSampler]=SMCSamplerForPaperv1(SMCSampler,Target,CurrentNbMCMCMoves,CurrentNbBlockMWG,Sensors,PercentageCESS,AdaptiveCoolingStrategy,PosteriorGrid,Xgrid,Ygrid);
% % % % %         else
% % % % %             SMCSampler.NbPart=CurrentNbParticles*NbIterationAveragedforIS;
% % % % %             [SMCSampler]=ImportanceSamplerForPaperv1(SMCSampler,Target,CurrentNbMCMCMoves,CurrentNbBlockMWG,Sensors,PercentageCESS,AdaptiveCoolingStrategy,PosteriorGrid,Xgrid,Ygrid);
% % % % %         end
% % % % % 
% % % % % 
% % % % % 
% % % % %         ResultsSMC.SavedLogBayesianEvidence(1,nMCruns)=SMCSampler.LogBayesEvidence;%LogBayesEvidence;
% % % % %         ResultsSMC.SavedEstimateBeta.Mean(:,nMCruns)=SMCSampler.Estimate.Mean;
% % % % %         
% % % % %         ResultsSMC.SavedEstimateBeta.Cov(:,:,nMCruns)=SMCSampler.Estimate.Cov; 
% % % % %         
% % % % %         
% % % % %         % Save all particles and weights of the last iteration
% % % % %         
% % % % %         ResultsSMC.MCruns(nMCruns).SavedNess=SMCSampler.EffectiveSampleSize; % Since Number of iterations could be different
% % % % %         ResultsSMC.SavedParticles.Beta(:,:,nMCruns)=[SMCSampler.Iter(end).Particles(:).Beta];
% % % % %         ResultsSMC.SavedParticles.LogPower(:,:,nMCruns)=[SMCSampler.Iter(end).Particles(:).LogPower];
% % % % %         ResultsSMC.SavedParticles.NormalizedWeights(1,:,nMCruns)=SMCSampler.NormalizedWeights(:,:,SMCSampler.NumIter);
% % % % %         
% % % % %         
% % % % %         ResultsSMC.MCruns(nMCruns).Temperature=SMCSampler.Temperature(1:SMCSampler.NumIter);
% % % % %         
% % % % %         ResultsSMC.MCruns(nMCruns).NbIter=SMCSampler.NumIter;
% % % % % 
% % % % %        TrueTargetPosition(:,nMCruns)=Target.Position;
% % % % %        TrueTargetPower(:,nMCruns)=Target.Power;
% % % % %     
% % % % % end


